@@ -21,7 +21,7 @@ module Bundler
             "Run `bundle install` to update and install the bundled gems."
         end
         # cycle through the requested gems, just to make sure they exist
-        names = Bundler.locked_gems.specs.map{ |s| s.name }
+        names = Bundler.locked_gems.specs.map(&:name)
         gems.each do |g|
           next if names.include?(g)
           require "bundler/cli/common"
@@ -30,7 +30,7 @@ module Bundler
 
         if groups.any?
           specs = Bundler.definition.specs_for groups
-          sources.concat(specs.map(&:name))
+          gems.concat(specs.map(&:name))
         end
 
         Bundler.definition(:gems => gems, :sources => sources)
@@ -49,14 +49,14 @@ module Bundler
 
       Bundler.definition.validate_ruby!
       Installer.install Bundler.root, Bundler.definition, opts
-      Bundler.load.cache if Bundler.root.join("vendor/cache").exist?
+      Bundler.load.cache if Bundler.app_cache.exist?
 
       if Bundler.settings[:clean] && Bundler.settings[:path]
         require "bundler/cli/clean"
         Bundler::CLI::Clean.new(options).run
       end
 
-      Bundler.ui.confirm "Your bundle is updated!"
+      Bundler.ui.confirm "Bundle updated!"
       without_groups_messages
     end
 
@@ -68,6 +68,5 @@ module Bundler
         Bundler.ui.confirm Bundler::CLI::Common.without_groups_message
       end
     end
-
   end
 end
