@@ -1,13 +1,12 @@
-require 'bundler/fetcher/base'
+require "bundler/fetcher/base"
+require "rubygems/remote_fetcher"
 
 module Bundler
   class Fetcher
     class Index < Base
       def specs(_gem_names)
-        old_sources = Bundler.rubygems.sources
-        Bundler.rubygems.sources = [remote_uri.to_s]
-        Bundler.rubygems.fetch_all_remote_specs
-      rescue Gem::RemoteFetcher::FetchError, OpenSSL::SSL::SSLError => e
+        Bundler.rubygems.fetch_all_remote_specs(remote)
+      rescue Gem::RemoteFetcher::FetchError, OpenSSL::SSL::SSLError, Net::HTTPFatalError => e
         case e.message
         when /certificate verify failed/
           raise CertificateFailureError.new(display_uri)
@@ -23,8 +22,6 @@ module Bundler
           Bundler.ui.trace e
           raise HTTPError, "Could not fetch specs from #{display_uri}"
         end
-      ensure
-        Bundler.rubygems.sources = old_sources
       end
     end
   end
