@@ -1,11 +1,10 @@
 module Bundler
   class Source
-
     class Path < Source
-      autoload :Installer, 'bundler/source/path/installer'
+      autoload :Installer, "bundler/source/path/installer"
 
-      attr_reader   :path, :options
-      attr_writer   :name
+      attr_reader :path, :options
+      attr_writer :name
       attr_accessor :version
 
       DEFAULT_GLOB = "{,*,*/*}.gemspec"
@@ -50,37 +49,38 @@ module Bundler
       end
 
       def to_s
-        "source at #{@path}"
+        "source at `#{@path}`"
       end
 
       def hash
-        self.class.hash
+        [self.class, expanded_path, version].hash
       end
 
-      def eql?(o)
-        o.instance_of?(Path) &&
-        expand(path) == expand(o.path) &&
-        version == o.version
+      def eql?(other)
+        other.instance_of?(Path) &&
+          expanded_path == expand(other.path) &&
+          version == other.version
       end
 
-      alias == eql?
+      alias_method :==, :eql?
 
       def name
-        File.basename(expand(path).to_s)
+        File.basename(expanded_path.to_s)
       end
 
-      def install(spec)
+      def install(spec, force = false)
+        Bundler.ui.info "Using #{version_message(spec)} from #{self}"
         generate_bin(spec, :disable_extensions)
-        ["Using #{version_message(spec)} from #{to_s}", nil]
+        nil # no post-install message
       end
 
       def cache(spec, custom_path = nil)
         app_cache_path = app_cache_path(custom_path)
         return unless Bundler.settings[:cache_all]
-        return if expand(@original_path).to_s.index(Bundler.root.to_s) == 0
+        return if expand(@original_path).to_s.index(Bundler.root.to_s + "/") == 0
 
         unless @original_path.exist?
-          raise GemNotFound, "Can't cache gem #{version_message(spec)} because #{to_s} is missing!"
+          raise GemNotFound, "Can't cache gem #{version_message(spec)} because #{self} is missing!"
         end
 
         FileUtils.rm_rf(app_cache_path)
@@ -95,6 +95,7 @@ module Bundler
       def specs
         if has_app_cache?
           @path = app_cache_path
+          @expanded_path = nil # Invalidate
         end
         local_specs
       end
@@ -105,6 +106,10 @@ module Bundler
 
     private
 
+      def expanded_path
+        @expanded_path ||= expand(path)
+      end
+
       def expand(somepath)
         somepath.expand_path(Bundler.root)
       rescue ArgumentError => e
@@ -123,16 +128,14 @@ module Bundler
 
       def load_spec_files
         index = Index.new
-        expanded_path = expand(path)
 
         if File.directory?(expanded_path)
-          Dir["#{expanded_path}/#{@glob}"].each do |file|
-            spec = Bundler.load_gemspec(file)
-            if spec
-              spec.loaded_from = file.to_s
-              spec.source = self
-              index << spec
-            end
+          # We sort depth-first since `<<` will override the earlier-found specs
+          Dir["#{expanded_path}/#{@glob}"].sort_by {|p| -p.split(File::SEPARATOR).size }.each do |file|
+            next unless spec = Bundler.load_gemspec(file, :validate)
+            spec.loaded_from = file.to_s
+            spec.source = self
+            index << spec
           end
 
           if index.empty? && @name && @version
@@ -143,11 +146,11 @@ module Bundler
               s.platform = Gem::Platform::RUBY
               s.summary  = "Fake gemspec for #{@name}"
               s.relative_loaded_from = "#{@name}.gemspec"
-              s.authors  = ["no one"]
+              s.authors = ["no one"]
               if expanded_path.join("bin").exist?
                 executables = expanded_path.join("bin").children
-                executables.reject!{|p| File.directory?(p) }
-                s.executables = executables.map{|c| c.basename.to_s }
+                executables.reject! {|p| File.directory?(p) }
+                s.executables = executables.map {|c| c.basename.to_s }
               end
             end
           end
@@ -161,14 +164,14 @@ module Bundler
       end
 
       def relative_path
-        if path.to_s.match(%r{^#{Regexp.escape Bundler.root.to_s}})
+        if path.to_s.start_with?(Bundler.root.to_s)
           return path.relative_path_from(Bundler.root)
         end
         path
       end
 
       def generate_bin(spec, disable_extensions = false)
-        gem_dir  = Pathname.new(spec.full_gem_path)
+        gem_dir = Pathname.new(spec.full_gem_path)
 
         # Some gem authors put absolute paths in their gemspec
         # and we have to save them from themselves
@@ -181,14 +184,14 @@ module Bundler
           end
         end.compact
 
-        gem_file = Bundler.rubygems.build_gem gem_dir, spec
-
-        installer = Path::Installer.new(spec, :env_shebang => false)
-        run_hooks(:pre_install, installer)
-        installer.build_extensions unless disable_extensions
-        run_hooks(:post_build, installer)
-        installer.generate_bin
-        run_hooks(:post_install, installer)
+        SharedHelpers.chdir(gem_dir) do
+          installer = Path::Installer.new(spec, :env_shebang => false)
+          run_hooks(:pre_install, installer)
+          installer.build_extensions unless disable_extensions
+          run_hooks(:post_build, installer)
+          installer.generate_bin
+          run_hooks(:post_install, installer)
+        end
       rescue Gem::InvalidSpecificationException => e
         Bundler.ui.warn "\n#{spec.name} at #{spec.full_gem_path} did not have a valid gemspec.\n" \
                         "This prevents bundler from installing bins or native extensions, but " \
@@ -201,10 +204,6 @@ module Bundler
         end
 
         Bundler.ui.warn "The validation message from Rubygems was:\n  #{e.message}"
-      ensure
-        if gem_dir && gem_file
-          FileUtils.rm_rf(gem_dir.join gem_file)
-        end
       end
 
       def run_hooks(type, installer)
@@ -212,14 +211,12 @@ module Bundler
         return unless Gem.respond_to?(hooks_meth)
         Gem.send(hooks_meth).each do |hook|
           result = hook.call(installer)
-          if result == false
-            location = " at #{$1}" if hook.inspect =~ /@(.*:\d+)/
-            message = "#{type} hook#{location} failed for #{installer.spec.full_name}"
-            raise InstallHookError, message
-          end
+          next unless result == false
+          location = " at #{$1}" if hook.inspect =~ /@(.*:\d+)/
+          message = "#{type} hook#{location} failed for #{installer.spec.full_name}"
+          raise InstallHookError, message
         end
       end
     end
-
   end
 end
