@@ -1,8 +1,7 @@
 module Bundler
   class Source
     class Path
-
-      class Installer < Bundler::GemInstaller
+      class Installer < Bundler::RubyGemsGemInstaller
         attr_reader :spec
 
         def initialize(spec, options = {})
@@ -28,7 +27,9 @@ module Bundler
           super
 
           if Bundler.requires_sudo?
-            Bundler.mkdir_p @gem_bin_dir
+            SharedHelpers.filesystem_access(@gem_bin_dir) do |p|
+              Bundler.mkdir_p(p)
+            end
             spec.executables.each do |exe|
               Bundler.sudo "cp -R #{@bin_dir}/#{exe} #{@gem_bin_dir}"
             end
@@ -37,7 +38,6 @@ module Bundler
           Bundler.rm_rf(@tmp_dir) if Bundler.requires_sudo?
         end
       end
-
     end
   end
 end
